@@ -6,6 +6,7 @@
  * calculation loops for various formulas/algorithms. It uses lots of
  * coprocesor magic. It is included from formulas.c
  */
+
 #ifndef VARIABLES		/*supply defaultd values */
 #define VARIABLES
 #endif
@@ -45,6 +46,38 @@
 #ifndef __i386__
 #undef USEHACKS
 #endif
+#ifdef NOASSEMBLY
+#undef USEHACKS
+#endif
+
+/* Prepare main loop */
+#ifndef NSFORMULALOOP
+#define NSFORMULALOOP(iter) \
+	  do \
+	    {			/*try first 8 iterations */ \
+	      FORMULA; \
+	      iter--; \
+	    } \
+	  while (BTEST && iter)
+#endif
+#ifndef SFORMULALOOP
+#define SFORMULALOOP(iter) \
+  	  do \
+	    {			/*try first 8 iterations */ \
+	      SAVEZMAG; \
+	      FORMULA; \
+	      iter--; \
+	    } \
+	  while (BTEST && iter)
+#endif
+#ifndef FORMULALOOP
+#ifdef SMOOTHMODE
+#define FORMULALOOP SFORMULALOOP
+#else
+#define FORMULALOOP NSFORMULALOOP
+#endif
+#endif
+
 #ifdef USEHACKS
 #ifdef RPIP
 #define I386HACK1 __asm__ ("#HACK1" : \
@@ -54,18 +87,12 @@
 		);
 #define I386HACK __asm__ ("#HACK" : \
 		: \
-		 "f" (rp), \
-		 "f" (ip), \
-		 "f" (pre), \
-		 "f" (pim), \
 		 "f" (zre), \
 		 "f" (zim) \
 		);
 #else
 #define I386HACK __asm__ ("#HACK" : \
 		: \
-		 "f" (pre), \
-		 "f" (pim), \
 		 "f" (zre), \
 		 "f" (zim) \
 		);
@@ -88,18 +115,24 @@
 #ifdef UNCOMPRESS
 /*uncompressed version of loop */
 #ifdef SMOOTHMODE
-static int FUNCTYPE
-SCALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int FUNCTYPE
-SCALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+SCALC (register number_t zre, register number_t zim, register number_t pre,
+       register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       SCALC (register number_t zre, register number_t zim,
+	      register number_t pre, register number_t pim)
 #else
-static int FUNCTYPE
-CALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int
-CALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+CALC (register number_t zre, register number_t zim, register number_t pre,
+      register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned
+       CALC (register number_t zre, register number_t zim,
+	     register number_t pre, register number_t pim)
 #endif
 {
-  register int iter = cfractalc.maxiter;
+  register unsigned int iter = cfractalc.maxiter;
   number_t szre = 0, szim = 0;
 #ifdef RPIP
   register number_t rp = 0, ip;
@@ -107,8 +140,7 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
 #ifdef SMOOTHMODE
   number_t szmag = 0;
 #endif
-  SAVEVARIABLES
-    VARIABLES;
+  SAVEVARIABLES VARIABLES;
   INIT;
   if (PRETEST)
     iter = 0;
@@ -122,36 +154,50 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
 	{
 	  I386HACK1;
 	  I386HACK;
-	  while (iter && BTEST)
-	    {			/*try first 8 iterations */
-	      SAVEZMAG;
-	      FORMULA;
-	      iter--;
+
+	  /*try first 8 iterations */
+	  if (BTEST && iter)
+	    {
+	      FORMULALOOP (iter);
 	    }
+	  /*
+	     while (BTEST && iter)
+	     {  
+	     SAVEZMAG;
+	     FORMULA;
+	     iter--;
+	     } */
 	}
       else
 	{
 	  iter = 8 + (cfractalc.maxiter & 7);
 	  I386HACK1;
 	  I386HACK;
-	  while (iter && BTEST)
-	    {			/*try first 8 iterations */
-	      SAVEZMAG;
-	      FORMULA;
-	      iter--;
+
+	  /*try first 8 iterations */
+	  if (BTEST && iter)
+	    {
+	      FORMULALOOP (iter);
 	    }
+	  /*
+	     while (BTEST && iter)
+	     {  
+	     SAVEZMAG;
+	     FORMULA;
+	     iter--;
+	     } */
 	  if (BTEST)
 	    {
 	      iter = (cfractalc.maxiter - 8) & (~7);
 	      iter >>= 3;
+	      I386HACK1;
+	      I386HACK;		/*do next 8 iteration w/o out of bounds checking */
 	      do
 		{
 		  /*hmm..we are probably in some deep area. */
 		  szre = zre;	/*save current possition */
 		  szim = zim;
 		  SAVE;
-		  I386HACK1;
-		  I386HACK;	/*do next 8 iteration w/o out of bounds checking */
 		  UFORMULA;
 		  UFORMULA;
 		  UFORMULA;
@@ -163,7 +209,7 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
 		  UEND;
 		  iter--;
 		}
-	      while (iter && BTEST);
+	      while (BTEST && iter);
 	      if (!(BTEST))
 		{		/*we got out of bounds */
 		  iter <<= 3;
@@ -177,13 +223,15 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
 #endif
 		  I386HACK1;
 		  I386HACK;
-		  do
-		    {
-		      SAVEZMAG
-			FORMULA;
-		      iter--;
-		    }
-		  while ((iter) && BTEST);
+		  FORMULALOOP (iter);
+		  /*
+		     do
+		     {
+		     SAVEZMAG
+		     FORMULA;
+		     iter--;
+		     }
+		     while (BTEST && iter); */
 		}
 	    }
 	  else
@@ -204,18 +252,24 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
 }
 #else
 #ifdef SMOOTHMODE
-static int FUNCTYPE
-SCALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int FUNCTYPE
-SCALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+SCALC (register number_t zre, register number_t zim, register number_t pre,
+       register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       SCALC (register number_t zre, register number_t zim,
+	      register number_t pre, register number_t pim)
 #else
-static int FUNCTYPE
-CALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int FUNCTYPE
-CALC (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+CALC (register number_t zre, register number_t zim, register number_t pre,
+      register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       CALC (register number_t zre, register number_t zim,
+	     register number_t pre, register number_t pim)
 #endif
 {
-  register int iter = cfractalc.maxiter /*& (~(int) 3) */ ;
+  register unsigned int iter = cfractalc.maxiter /*& (~(int) 3) */ ;
 #ifdef RPIP
   register number_t rp, ip;
 #endif
@@ -232,14 +286,19 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
       rp = zre * zre;
       ip = zim * zim;
 #endif
-      while ((iter) && BTEST)
+      if (BTEST && iter)
 	{
-	  I386HACK;
-	  SAVEZMAG
-	    FORMULA;
-	  iter--;
-
+	  FORMULALOOP (iter);
 	}
+      /*
+         while (BTEST && iter)
+         {
+         I386HACK;
+         SAVEZMAG
+         FORMULA;
+         iter--;
+
+         } */
     }
 #ifdef SMOOTHMODE
   if (iter)
@@ -280,24 +339,30 @@ CALC (register number_t zre, register number_t zim, register number_t pre, regis
    But this should be calculated outter from here (ie each frame, for example), to avoid
    new calculs */
 #ifdef PERI
-#define PCHECK (myabs(r1 - zre)<cfractalc.periodicity_limit && myabs(s1 - zim) < cfractalc.periodicity_limit)
+#define PCHECK (abs_less_than(r1 - zre, cfractalc.periodicity_limit) && abs_less_than(s1 - zim, cfractalc.periodicity_limit))
 
 #ifndef UNCOMPRESS
 
 #ifdef SMOOTHMODE
-static int FUNCTYPE
-SPERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int FUNCTYPE
-SPERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+SPERI (register number_t zre, register number_t zim, register number_t pre,
+       register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       SPERI (register number_t zre, register number_t zim,
+	      register number_t pre, register number_t pim)
 #else
-static int FUNCTYPE
-PERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
-CONSTF REGISTERS (3);
-static int FUNCTYPE
-PERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+PERI (register number_t zre, register number_t zim, register number_t pre,
+      register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       PERI (register number_t zre, register number_t zim,
+	     register number_t pre, register number_t pim)
 #endif
 {
-  register int iter = cfractalc.maxiter /*& (~(int) 3) */ , iter1 = 8;
+  register unsigned int iter = cfractalc.maxiter /*& (~(int) 3) */ , iter1 =
+    8;
   register number_t r1, s1;
   int whensavenew, whenincsave;
 #ifdef RPIP
@@ -319,12 +384,19 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
       I386HACK;
       if (iter < iter1)
 	iter1 = iter, iter = 8;
-      while (iter1 && BTEST)
-	{			/*H. : do first few iterations w/o checking */
-	  SAVEZMAG;
-	  FORMULA;
-	  iter1--;
+
+      /*H. : do first few iterations w/o checking */
+      if (BTEST && iter1)
+	{
+	  FORMULALOOP (iter1);
 	}
+      /*
+         while (BTEST && iter1)
+         {      
+         SAVEZMAG;
+         FORMULA;
+         iter1--;
+         } */
       if (iter1)
 	{
 	  if (iter >= 8)
@@ -347,7 +419,7 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 	  /*F. : problem is that after deep zooming, peiodicity is never detected early, cause is is
 	     quite slow before going in a periodic loop.
 	     So, we should start checking periodicity only after some times */
-	  while ((iter) && BTEST)
+	  while (BTEST && iter)
 	    {
 	      SAVEZMAG;
 	      FORMULA;
@@ -395,29 +467,34 @@ end:
 /*F. : this macros definitions are really strange, but after a while, it's good */
 
 #ifdef SMOOTHMODE
-static int FUNCTYPE
-SPERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int FUNCTYPE
-SPERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+SPERI (register number_t zre, register number_t zim, register number_t pre,
+       register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       SPERI (register number_t zre, register number_t zim,
+	      register number_t pre, register number_t pim)
 #else
-static int FUNCTYPE
-PERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim) CONSTF REGISTERS (3);
-static int FUNCTYPE
-PERI (register number_t zre, register number_t zim, register number_t pre, register number_t pim)
+static unsigned int FUNCTYPE
+PERI (register number_t zre, register number_t zim, register number_t pre,
+      register number_t pim)
+     CONSTF REGISTERS (3);
+     static unsigned int FUNCTYPE
+       PERI (register number_t zre, register number_t zim,
+	     register number_t pre, register number_t pim)
 #endif
 {
-  register int iter = cfractalc.maxiter /*& (~(int) 3) */ ;
+  register unsigned int iter = cfractalc.maxiter /*& (~(int) 3) */ ;
   register number_t r1 = zre, s1 = zim;
   number_t szre = 0, szim = 0;	/*F. : Didn't declared register, cause they are few used */
-  int whensavenew, whenincsave;
+  unsigned int whensavenew, whenincsave;
 #ifdef RPIP
   register number_t rp = 0, ip;
 #endif
 #ifdef SMOOTHMODE
   number_t szmag = 0;
 #endif
-  SAVEVARIABLES
-    VARIABLES;
+  SAVEVARIABLES VARIABLES;
   INIT;
   if (PRETEST)
     iter = 0;
@@ -431,12 +508,18 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 	  rp = zre * zre;
 	  ip = zim * zim;
 #endif
-	  while (iter && BTEST)
-	    {			/*F. : Added iter&7 to be sure we'll be on a 8 multiple */
-	      SAVEZMAG
-		FORMULA;
-	      iter--;
+	  /*F. : Added iter&7 to be sure we'll be on a 8 multiple */
+	  if (BTEST && iter)
+	    {
+	      FORMULALOOP (iter);
 	    }
+	  /*
+	     while (BTEST && iter)
+	     {
+	     SAVEZMAG
+	     FORMULA;
+	     iter--;
+	     } */
 	}
       else
 	{
@@ -454,10 +537,9 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 	  I386HACK1;
 	  /*I386HACK; */
 	  iter = 8 + (cfractalc.maxiter & 7);
-	  while (iter && BTEST)
+	  while (BTEST && iter)
 	    {			/*F. : Added iter&7 to be sure we'll be on a 8 multiple */
-	      SAVEZMAG
-		FORMULA;
+	      SAVEZMAG FORMULA;
 	      iter--;
 	    }
 	  if (BTEST)
@@ -469,7 +551,7 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 		  szre = zre, szim = zim;
 		  SAVE;
 		  SAVEZMAG
-		  /*I386HACK; */
+		    /*I386HACK; */
 		    I386HACK1;
 		  FORMULA;	/*F. : Calculate one time */
 		  if (PCHECK)
@@ -508,7 +590,7 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 			}
 		    }
 		}
-	      while (iter && BTEST);
+	      while (BTEST && iter);
 	      if (!BTEST)
 		{		/*we got out of bounds */
 		  iter += 8;	/*restore saved possition */
@@ -521,13 +603,15 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 #endif
 		  I386HACK1;
 		  /*I386HACK; */
-		  do
-		    {
-		      SAVEZMAG
-			FORMULA;
-		      iter--;
-		    }
-		  while ((iter) && BTEST);
+		  FORMULALOOP (iter);
+		  /*
+		     do
+		     {
+		     SAVEZMAG
+		     FORMULA;
+		     iter--;
+		     }
+		     while (BTEST && iter); */
 		}
 	    }
 	  else
@@ -548,6 +632,7 @@ PERI (register number_t zre, register number_t zim, register number_t pre, regis
 periodicity:
   PERIINOUTPUT ();
 }
+
 /*else uncompress */
 #endif
 
@@ -558,10 +643,10 @@ periodicity:
 #ifndef SMOOTHMODE
 #ifdef JULIA
 static void
-JULIA (struct image *image, register number_t pre, register number_t pim, int maxiter)
+JULIA (struct image *image, register number_t pre, register number_t pim)
 {
-  int i, i1, i2, j, x, y, itmp, itmp2;
-  int iter;
+  int i, i1, i2, j, x, y;
+  unsigned char iter, itmp2, itmp;
   number_t rp = 0, ip = 0;
   register number_t zre, zim, im, xdelta, ydelta, range, rangep;
   number_t xstep, ystep;
@@ -572,7 +657,7 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
   int guessed = 0, unguessed = 0, iters = 0;
 #endif
   VARIABLES;
-  range = RANGE;
+  range = (number_t) RANGE;
   rangep = range * range;
 
   xdelta = image->width / (RMAX - RMIN);
@@ -601,8 +686,9 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 	    if (y > 0 && y < image->height - 1 && *(addr + 1) &&
 		x > 0 && x < image->width - 1)
 	      {
-		if ((iter = *(addr + 1)) != NOT_CALCULATED && iter == *(addr - 1) && iter ==
-		    addr1[y - 1][x] && iter == addr1[y + 1][x])
+		if ((iter = *(addr + 1)) != NOT_CALCULATED
+		    && iter == *(addr - 1) && iter == addr1[y - 1][x]
+		    && iter == addr1[y + 1][x])
 		  {
 		    *addr = *(addr + 1);
 		    continue;
@@ -610,7 +696,7 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 	      }
 	    zim = im;
 	    zre = RMIN + (j + 0.5) * xstep;
-	    iter = 0;
+	    iter = (unsigned char) 0;
 	    qptr = queue;
 	    ip = (zim * zim);
 	    rp = (zre * zre);
@@ -619,11 +705,21 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 	      {
 		if (*addr != NOT_CALCULATED
 #ifdef SAG
-		    && (*addr == INPROCESS || (*addr != 1 &&
-				  (itmp2 = *(addr + 1)) != NOT_CALCULATED &&
-		((itmp2 != (itmp = *(addr - 1)) && itmp != NOT_CALCULATED) ||
-		 (itmp2 != (itmp = *((addr1[y + 1]) + x)) && itmp != NOT_CALCULATED) ||
-		 (itmp2 != (itmp = *((addr1[y - 1]) + x)) && itmp != NOT_CALCULATED))))
+		    && (*addr == INPROCESS || (*addr != (unsigned char) 1 &&
+					       (itmp2 =
+						*(addr + 1)) != NOT_CALCULATED
+					       &&
+					       ((itmp2 != (itmp = *(addr - 1))
+						 && itmp != NOT_CALCULATED)
+						|| (itmp2 !=
+						    (itmp =
+						     *((addr1[y + 1]) + x))
+						    && itmp != NOT_CALCULATED)
+						|| (itmp2 !=
+						    (itmp =
+						     *((addr1[y - 1]) + x))
+						    && itmp !=
+						    NOT_CALCULATED))))
 #endif
 		  )
 		  {
@@ -654,13 +750,14 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 		FORMULA;
 		ip = (zim * zim);
 		rp = (zre * zre);
-		if (rp + ip > RANGE || !(BTEST))
+		if (greater_than (rp + ip, RANGE) || !(BTEST))
 		  goto outset;
-		x = (zre - RMIN) * xdelta;
-		y = (zim - IMIN) * ydelta;
+		x = (int) ((zre - RMIN) * xdelta);
+		y = (int) ((zim - IMIN) * ydelta);
 		addr = addr1[y] + x;
-		if ((itmp = *(addr + 1)) != NOT_CALCULATED && itmp == *(addr - 1) && itmp ==
-		    addr1[y - 1][x] && itmp == addr1[y + 1][x])
+		if ((itmp = *(addr + 1)) != NOT_CALCULATED
+		    && itmp == *(addr - 1) && itmp == addr1[y - 1][x]
+		    && itmp == addr1[y + 1][x])
 		  {
 		    *addr = *(addr + 1);
 		  }
@@ -678,14 +775,15 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 	      {
 		qptr--;
 		iter++;
-		if (iter >= y)
-		  iter = 1;
+		if ((int) iter >= y)
+		  iter = (unsigned char) 1;
 		**qptr = iter;
 	      }
 	  }
       }
 #ifdef STATISTICS
-  printf ("guessed %i, unguessed %i, iterations %i\n", guessed, unguessed, iters);
+  printf ("guessed %i, unguessed %i, iterations %i\n", guessed, unguessed,
+	  iters);
   guessed2 += guessed;
   unguessed2 += unguessed;
   iters2 += iters;
@@ -694,6 +792,7 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 #endif
 #endif
 
+#undef FORMULALOOP
 #undef PCHECK
 #undef I386HACK
 #undef I386HACK1
@@ -706,6 +805,8 @@ JULIA (struct image *image, register number_t pre, register number_t pim, int ma
 #endif
 
 /*cleanup for next formula */
+#undef NSFORMULALOOP
+#undef SFORMULALOOP
 #undef PRESMOOTH
 #undef SMOOTH
 #undef SMOOTHMODE
