@@ -5,6 +5,7 @@
 #include <aalib.h>
 #include <ui.h>
 #include <unistd.h>
+#include <xmenu.h>
 #include <archaccel.h>
 struct ui_driver aalib_driver;
 static aa_palette palette;
@@ -15,32 +16,46 @@ static aa_context *c;
 static unsigned char *secondary;
 static int mouse;
 static int nomouse;
+static void aa_build_menus (void);
+static void aa_remove_menus (void);
+extern unsigned char *aa_chardata;
+extern unsigned char *aa_colordata;
+extern int aa_cursorx, aa_cursory;
 
 
 
 static void
-aa_print (int x, int y, char *text)
+aa_print (int x, int y, CONST char *text)
 {
   aa_puts (c, x / 2, y / 2, AA_SPECIAL, text);
 }
 
 static void
-aa_display ()
+aa_display (void)
 {
-  aa_renderpalette (c, useaapalette ? aapalette : palette, &aa_defrenderparams, 0, 0, aa_scrwidth (c), aa_scrheight (c));
+  int i;
+  aa_renderpalette (c, useaapalette ? aapalette : palette,
+		    &aa_defrenderparams, 0, 0, aa_scrwidth (c),
+		    aa_scrheight (c));
+  for (i = 0; i < aa_scrwidth (c) * aa_scrheight (c); i++)
+    {
+      if (aa_colordata[i] != 255)
+	aa_text (c)[i] = aa_chardata[i], aa_attrs (c)[i] = aa_colordata[i];
+    }
 }
 static void
 aa_set_palette (ui_palette pal, int start, int end)
 {
   int i;
   for (i = start; i <= end; i++)
-    aa_setpalette (palette, i, pal[i - start][0], pal[i - start][1], pal[i - start][2]);
+    aa_setpalette (palette, i, pal[i - start][0], pal[i - start][1],
+		   pal[i - start][2]);
   aa_display ();
   aa_flush (c);
 }
 
 static void
-aa_flip_buffers ()
+aa_flip_buffers (void)
 {
   unsigned char *tmp;
   tmp = secondary;
@@ -51,6 +66,8 @@ aa_flip_buffers ()
 static void
 aa_free_buffers (char *b1, char *b2)
 {
+  free (aa_chardata);
+  free (aa_colordata);
   free (secondary);
 }
 
@@ -58,8 +75,10 @@ static int
 aa_alloc_buffers (char **b1, char **b2)
 {
   secondary = malloc (aa_imgwidth (c) * aa_imgheight (c));
-  *(unsigned char **)b2 = secondary;
-  *(unsigned char **)b1 = c->imagebuffer;
+  *(unsigned char **) b2 = secondary;
+  *(unsigned char **) b1 = c->imagebuffer;
+  aa_chardata = malloc (aa_scrwidth (c) * aa_scrheight (c));
+  aa_colordata = malloc (aa_scrwidth (c) * aa_scrheight (c));
   return aa_imgwidth (c);	/* bytes per scanline */
 }
 
@@ -79,12 +98,16 @@ aa_processevents (int wait, int *mx, int *my, int *mb, int *k)
   static int keys;
   do
     {
-      if ((ch = aa_getevent (c, wait)) != AA_NONE && ch != AA_MOUSE && ch != AA_RESIZE && ch < 256)
+      if ((ch = aa_getevent (c, wait)) != AA_NONE && ch != AA_MOUSE
+	  && ch != AA_RESIZE && ch < 256)
 	ui_key (ch);
       switch (ch)
 	{
 	case AA_BACKSPACE:
-	  ui_key (8);
+	  ui_key (UIKEY_BACKSPACE);
+	  break;
+	case AA_ESC:
+	  ui_key (UIKEY_ESC);
 	  break;
 	case AA_RESIZE:
 	  ui_resize ();
@@ -93,45 +116,33 @@ aa_processevents (int wait, int *mx, int *my, int *mb, int *k)
 	  aa_getmouse (c, &mousex, &mousey, &mouseb);
 	  break;
 	case AA_LEFT:
+	  ui_key (UIKEY_LEFT);
 	  if (c->kbddriver->flags & AA_SENDRELEASE)
 	    keys |= 1;
-	  else if (keys & 2)
-	    keys &= ~2;
-	  else
-	    keys ^= 1;
 	  break;
 	case AA_LEFT | AA_RELEASE:
 	  keys &= ~1;
 	  break;
 	case AA_RIGHT:
+	  ui_key (UIKEY_RIGHT);
 	  if (c->kbddriver->flags & AA_SENDRELEASE)
 	    keys |= 2;
-	  else if (keys & 1)
-	    keys &= ~1;
-	  else
-	    keys ^= 2;
 	  break;
 	case AA_RIGHT | AA_RELEASE:
 	  keys &= ~2;
 	  break;
 	case AA_UP:
+	  ui_key (UIKEY_UP);
 	  if (c->kbddriver->flags & AA_SENDRELEASE)
 	    keys |= 4;
-	  else if (keys & 8)
-	    keys &= ~8;
-	  else
-	    keys ^= 4;
 	  break;
 	case AA_UP | AA_RELEASE:
 	  keys &= ~4;
 	  break;
 	case AA_DOWN:
+	  ui_key (UIKEY_DOWN);
 	  if (c->kbddriver->flags & AA_SENDRELEASE)
 	    keys |= 8;
-	  else if (keys & 4)
-	    keys &= ~4;
-	  else
-	    keys ^= 8;
 	  break;
 	case AA_DOWN | AA_RELEASE:
 	  keys &= ~8;
@@ -166,9 +177,14 @@ static float aa_gamma, dimmul, boldmul;
 static int dithering[3];
 static int randomval = 0;
 static int masks[] =
-{AA_NORMAL_MASK, AA_DIM_MASK, AA_BOLD_MASK, AA_BOLDFONT_MASK, AA_REVERSE_MASK, AA_ALL, AA_EIGHT};
+  { AA_NORMAL_MASK, AA_DIM_MASK, AA_BOLD_MASK, AA_BOLDFONT_MASK,
+  AA_REVERSE_MASK, AA_ALL, AA_EIGHT
+};
+#ifdef DESTICKY
+extern int euid, egid;
+#endif
 static int
-aa_initialize ()
+aa_initialize (void)
 {
   int i, y;
   aa_parseoptions (NULL, NULL, NULL, NULL);	/*parse environment first */
@@ -177,7 +193,8 @@ aa_initialize ()
     {
       for (y = 0; aa_fonts[y] != NULL; y++)
 	{
-	  if (!strcmp (deffont, aa_fonts[y]->name) || !strcmp (deffont, aa_fonts[y]->shortname))
+	  if (!strcmp (deffont, aa_fonts[y]->name)
+	      || !strcmp (deffont, aa_fonts[y]->shortname))
 	    {
 	      aa_defparams.font = aa_fonts[y];
 	      break;
@@ -238,7 +255,16 @@ aa_initialize ()
   if (inverse)
     aa_defrenderparams.inversion = 1;
 
+#ifdef DESTICKY
+  seteuid (euid);		/* We need supervisor rights to open mouse. */
+  setegid (egid);
+#endif
   c = aa_autoinit (&aa_defparams);
+#ifdef DESTICKY
+  seteuid (getuid ());		/* Don't need supervisor rights anymore. */
+  setegid (getgid ());
+#endif
+
   if (c == NULL)
     return 0;
   aa_autoinitkbd (c, AA_SENDRELEASE);
@@ -247,13 +273,15 @@ aa_initialize ()
 
   aalib_driver.width = aa_mmwidth (c) / 10.0;
   aalib_driver.height = aa_mmheight (c) / 10.0;
+  aa_build_menus ();
   return (1);
 }
 
 static void
-aa_uninitialise ()
+aa_uninitialise (void)
 {
   aa_close (c);
+  aa_remove_menus ();
 }
 
 static void
@@ -272,13 +300,6 @@ aa_get_mouse (int *x, int *y, int *b)
     *b |= BUTTON3;
 }
 
-static void
-aa_clearscreen ()
-{
-  memset (c->textbuffer, ' ', aa_scrwidth (c) * aa_scrheight (c));
-  memset (c->attrbuffer, AA_NORMAL, aa_scrwidth (c) * aa_scrheight (c));
-}
-
 static void
 aa_mousetype (int type)
 {
@@ -290,91 +311,20 @@ aa_mousetype (int type)
 #endif
 }
 
-static char *name[] =
-{
-  "normal characters     ",
-  "half bright(dim)      ",
-  "double bright(bold)   ",
-  "bold font             ",
-  "reversed              ",
-  "reserved characters   ",
-  "non ascii characters  ",
-};
 
 #if AA_LIB_VERSION==1 && AA_LIB_MINNOR==0
 #define SUPPORTED c->driver->params.supported
 #else
 #define SUPPORTED c->driverparams.supported
 #endif
-static void
-aa_atributes (int m)
-{
-  static int mask;
-  char text[NATTRS + 1][50];
-  char *menu[NATTRS + 1];
-  int i;
-  switch (m)
-    {
-    case -1:
-      mask = c->params.supported;
-      break;
-    case 0:
-      mask ^= AA_NORMAL_MASK;
-      break;
-    case 1:
-      mask ^= AA_DIM_MASK;
-      break;
-    case 2:
-      mask ^= AA_BOLD_MASK;
-      break;
-    case 3:
-      mask ^= AA_BOLDFONT_MASK;
-      break;
-    case 4:
-      mask ^= AA_REVERSE_MASK;
-      break;
-    case 5:
-      mask ^= AA_ALL;
-      break;
-    case 6:
-      mask ^= AA_EIGHT;
-      break;
-    case 7:
-      return;
-    }
-  aa_setsupported (c, mask);
-  for (i = 0; i < NATTRS; i++)
-    {
-      menu[i] = &text[i][0];
-      sprintf (text[i], "%s %s", name[i], (SUPPORTED & masks[i]) ? (mask & masks[i]) ? "on" : "off" : "unsupported");
-    }
-  menu[i] = &text[i][0];
-  sprintf (text[i], "Close menu");
-  ui_menu (menu, "Available attributes:", NATTRS + 1, aa_atributes);
-}
 
-static void
-aa_sfont (int i)
-{
-  aa_setfont (c, aa_fonts[i]);
-}
-
-static void
-aa_font_menu (void)
-{
-  char *menu[100];
-  int i;
-  for (i = 0; aa_fonts[i] != NULL; i++)
-    menu[i] = aa_fonts[i]->name;
-  ui_menu (menu, "Please select font", i, aa_sfont);
-}
 
 static int driver;
 static int font;
 static int mask;
 
 static void
-aa_save5 (int m)
+aa_save5 (void)
 {
   struct aa_hardware_params p = aa_defparams;
   struct aa_context *sc;
@@ -382,15 +332,15 @@ aa_save5 (int m)
   int i;
   char *name;
   char extension[100];
-  if(aa_formats[driver]->flags&AA_USE_PAGES)
-  strcpy(extension,"_0_0");
+  if (aa_formats[driver]->flags & AA_USE_PAGES)
+    strcpy (extension, "_0_0");
   else
-  extension[0]=0;
-  strcat(extension,aa_formats[driver]->extension);
+    extension[0] = 0;
+  strcat (extension, aa_formats[driver]->extension);
 
 #ifdef DJGPP
   name = ui_getfile ("fr", extension);
-	  /*fit into ugly 8char. limit*/
+  /*fit into ugly 8char. limit */
 #else
   name = ui_getfile ("fract", extension);
 #endif
@@ -402,17 +352,17 @@ aa_save5 (int m)
       sleep (3);
       return;
     }
-  for (i = 0; name[i] != '.'&&name[i]!='_'; i++);
+  for (i = 0; name[i] != '.' && name[i] != '_'; i++);
   name[i] = 0;
-  strcat(name,"%c%e");
+  strcat (name, "%c%e");
 
   p.minwidth = p.maxwidth = 0;
   p.minheight = p.maxheight = 0;
   p.width = aa_scrwidth (c);
   p.height = aa_scrheight (c);
   p.supported = mask;
-  if (aa_formats[m]->font == NULL)
-    p.font = aa_fonts[m];
+  if (aa_formats[driver]->font == NULL)
+    p.font = aa_fonts[font];
 
   data.format = aa_formats[driver];
   data.name = name;
@@ -424,115 +374,28 @@ aa_save5 (int m)
       sleep (3);
       return;
     }
-  memcpy (sc->imagebuffer, c->imagebuffer, (aa_imgwidth (c) - 1) * aa_imgheight (c));
-  aa_renderpalette (sc, useaapalette ? aapalette : palette, &aa_defrenderparams, 0, 0, aa_scrwidth (c), aa_scrheight (c));
+  memcpy (sc->imagebuffer, c->imagebuffer,
+	  (aa_imgwidth (c) - 1) * aa_imgheight (c));
+  aa_renderpalette (sc, useaapalette ? aapalette : palette,
+		    &aa_defrenderparams, 0, 0, aa_scrwidth (c),
+		    aa_scrheight (c));
   aa_flush (sc);
   aa_close (sc);
 }
 
 static void
-aa_save4 (int m)
+aa_swinversion (struct uih_context *c)
 {
-  char text[NATTRS + 1][50];
-  char *menu[NATTRS + 1];
-  int i;
-  switch (m)
-    {
-    case -1:
-      mask = aa_formats[driver]->supported & c->params.supported;
-      if (!mask)
-	mask = aa_formats[driver]->supported;
-      break;
-    case 0:
-      mask ^= AA_NORMAL_MASK;
-      break;
-    case 1:
-      mask ^= AA_DIM_MASK;
-      break;
-    case 2:
-      mask ^= AA_BOLD_MASK;
-      break;
-    case 3:
-      mask ^= AA_BOLDFONT_MASK;
-      break;
-    case 4:
-      mask ^= AA_REVERSE_MASK;
-      break;
-    case 5:
-      mask ^= AA_ALL;
-      break;
-    case 6:
-      mask ^= AA_EIGHT;
-      break;
-    case 7:
-      aa_save5 (font);
-      return;
-    }
-  for (i = 0; i < NATTRS; i++)
-    {
-      menu[i] = &text[i][0];
-      sprintf (text[i], "%s %s", name[i], (aa_formats[driver]->supported & masks[i]) ? (mask & masks[i]) ? "on" : "off" : "unsupported");
-    }
-  menu[i] = &text[i][0];
-  sprintf (text[i], "save file");
-  ui_menu (menu, "Available attributes:", NATTRS + 1, aa_save4);
+  aa_defrenderparams.inversion ^= 1;
 }
-
-static void
-aa_save3 (int m)
+static int
+aa_inversion (struct uih_context *c)
 {
-  aa_display ();
-  font = m;
-  aa_save4 (-1);
+  return (aa_defrenderparams.inversion);
 }
-
+static int aacurpalette = 0;
 static void
-aa_save2 (int m)
-{
-  static char *menu[100];
-  int i;
-
-  driver = m;
-  if (aa_formats[m]->font != NULL)
-    aa_save3 (0);
-  else
-    {
-      for (i = 0; aa_fonts[i] != NULL; i++)
-	menu[i] = aa_fonts[i]->name;
-      ui_menu (menu, "Please select font", i, aa_save3);
-    }
-}
-
-static void
-aa_save (void)
-{
-  char *menu[100];
-  int i;
-  for (i = 0; aa_formats[i] != NULL; i++)
-    menu[i] = aa_formats[i]->formatname;
-  ui_menu (menu, "Please select format", i, aa_save2);
-}
-
-static void
-aa_dither (int m)
-{
-  aa_defrenderparams.dither = m;
-}
-
-static void
-aa_dither_menu (void)
-{
-  char *menu[100];
-  int i;
-
-  aa_display ();
-  for (i = 0; i < AA_DITHERTYPES; i++)
-    menu[i] = aa_dithernames[i];
-  ui_menu (menu, "Please select dithering mode", i, aa_dither);
-}
-
-static void
-aa_spalette (int m)
+aa_setXpalette (struct uih_context *c, int m)
 {
   int i, s;
   if (!m)
@@ -542,6 +405,7 @@ aa_spalette (int m)
       return;
     }
   useaapalette = 1;
+  aacurpalette = m;
   s = 1 << m;
   s--;
   for (i = 0; i < 255; i++)
@@ -549,150 +413,240 @@ aa_spalette (int m)
       aapalette[i] = (i & (s)) * 255 / s;
     }
 }
-
-static void
-aa_palette_menu (void)
+static int
+aa_getpalette (struct uih_context *c, int m)
 {
-  static char *menu[] =
-  {
-    "Use XaoS palette",
-    "Black and white stripes",
-    "4 gray palette",
-    "16 gray palette",
-    "32 gray palette",
-    "64 gray palette",
-    "128 gray palette",
-    "256 gray palette"
-  };
-  ui_menu (menu, "Please select AA palette", sizeof (menu) / sizeof (char *), aa_spalette);
+  if (!useaapalette)
+    return (!m);
+  return (m == aacurpalette);
+}
+static void
+aa_dither (struct uih_context *c, int m)
+{
+  aa_defrenderparams.dither = m;
+}
+static int
+aa_getdither (struct uih_context *c, int mode)
+{
+  return ((int) aa_defrenderparams.dither == (int) mode);
 }
 
+
 static void
-aa_actionpress (int m)
+aa_sfont (struct uih_context *co, int i)
 {
-  switch (m)
+  aa_setfont (c, aa_fonts[i]);
+}
+CONST static char *CONST name[] = {
+  "normal characters     ",
+  "half bright(dim)      ",
+  "double bright(bold)   ",
+  "bold font             ",
+  "reversed              ",
+  "reserved characters   ",
+  "non ascii characters  ",
+  "leave menu",
+};
+CONST static char *CONST ttext[] = {
+  "Use XaoS palette",
+  "Black and white stripes",
+  "4 gray palette",
+  "16 gray palette",
+  "32 gray palette",
+  "64 gray palette",
+  "128 gray palette",
+  "256 gray palette"
+};
+static CONST int attribs[] = {
+  AA_NORMAL_MASK,
+  AA_DIM_MASK,
+  AA_BOLD_MASK,
+  AA_BOLDFONT_MASK,
+  AA_REVERSE_MASK,
+  AA_ALL,
+  AA_EIGHT,
+  0
+};
+#define MYNATTRS (sizeof(name)/sizeof(char *))
+static void
+aa_swattr (struct uih_context *co, int m)
+{
+  int mask;
+  if (m < (int) MYNATTRS - 1)
     {
-    case 0:
-      aa_atributes (-1);
-      break;
-    case 1:
-      aa_font_menu ();
-      break;
-    case 2:
-      aa_defrenderparams.inversion ^= 1;
-      break;
-    case 3:
-      aa_dither_menu ();
-      break;
-    case 4:
-      aa_palette_menu ();
-      break;
-    case 5:
-      aa_save ();
-      break;
+      mask = c->params.supported;
+      mask ^= attribs[m];
+      aa_setsupported (c, mask);
+      ui_menu ("aa_attr");
     }
 }
-
-static void
-aa_action (void)
+static int
+aa_getattr (struct uih_context *co, int m)
 {
-  static char *menu[] =
-  {
-    "Attributes",
-    "Font",
-    "Inversion",
-    "Dithering mode",
-    "Palette options",
-    "Save text screen",
-  };
-  ui_menu (menu, "AA driver options", sizeof (menu) / sizeof (char *), aa_actionpress);
+  return (c->params.supported & attribs[m]);
 }
+static int
+aa_getsave3 (struct uih_context *c, int m)
+{
+  return (mask & attribs[m]);
+}
+static void
+aa_save3 (struct uih_context *co, int m)
+{
+  if (m < (int) MYNATTRS - 1)
+    {
+      mask ^= attribs[m];
+      mask &= aa_formats[driver]->supported;
+      ui_menu ("aa_save3");
+    }
+  else
+    aa_save5 ();
+}
+static void
+aa_save2 (struct uih_context *cc, int m)
+{
+  aa_display ();
+  font = m;
+  mask = aa_formats[driver]->supported & c->params.supported;
+  if (!mask)
+    mask = aa_formats[driver]->supported;
+  ui_menu ("aa_save3");
+}
+static void
+aa_save (struct uih_context *c, int m)
+{
+  driver = m;
+  if (aa_formats[m]->font != NULL)
+    aa_save3 (NULL, 0);
+  else
+    ui_menu ("aa_save2");
+}
+
+#define UI (MENUFLAG_NOOPTION|MENUFLAG_NOPLAY)
+static CONST menuitem menuitems[] = {
+  SUBMENU ("file", NULL, "Save as text file", "aa_format"),
+  SUBMENU ("ui", NULL, "Attributes", "aa_attr"),
+  SUBMENU ("ui", NULL, "Font", "aa_font"),
+  MENUNOPCB ("ui", NULL, "Inversion", "aainversion", UI, aa_swinversion,
+	     aa_inversion),
+  SUBMENU ("ui", NULL, "Dithering mode", "aa_dithering"),
+  SUBMENU ("", NULL, "Font for saved file", "aa_save2"),
+  SUBMENU ("", NULL, "Save attributes", "aa_save3"),
+  SUBMENU ("palettemenu", NULL, "Text palette", "aa_palette"),
+};
+
+static menuitem *fontmenus;
+static menuitem *fontmenus2;
+static menuitem *formatmenus;
+static int nfonts, nformats;
+static void
+aa_build_menus ()
+{
+  int i;
+  menu_add (menuitems, NITEMS (menuitems));
+  menu_genernumbered (sizeof (ttext) / sizeof (char *), "aa_palette", ttext,
+		      NULL, MENU_INT, MENUFLAG_RADIO, aa_setXpalette,
+		      aa_getpalette, "palettemenu");
+  menu_genernumbered (AA_DITHERTYPES, "aa_dithering",
+		      (CONST char *CONST * CONST) aa_dithernames, NULL,
+		      MENU_INT, MENUFLAG_RADIO, aa_dither, aa_getdither,
+		      "dither");
+  menu_genernumbered (MYNATTRS, "aa_attr", name, NULL, MENU_INT,
+		      MENUFLAG_RADIO, aa_swattr, aa_getattr, "attribute");
+  menu_genernumbered (MYNATTRS, "aa_save3", name, NULL, MENU_INT,
+		      MENUFLAG_RADIO, aa_save3, aa_getsave3, "save");
+  for (i = 0; aa_fonts[i] != NULL; i++);
+  nfonts = i;
+  fontmenus = malloc (sizeof (menuitem) * i);
+  fontmenus2 = malloc (sizeof (menuitem) * i);
+  for (i = 0; aa_fonts[i] != NULL; i++)
+    {
+      char s[256];
+      sprintf (s, "font%i%s", i, aa_fonts[i]->shortname);
+      fontmenus[i].name = aa_fonts[i]->name;
+      fontmenus[i].shortname = strdup (s);
+      fontmenus[i].menuname = "aa_font";
+      fontmenus[i].key = NULL;
+      fontmenus[i].type = MENU_INT;
+      fontmenus[i].flags = UI;
+      fontmenus[i].iparam = i;
+      fontmenus[i].function = (void (*)(void)) aa_sfont;
+      fontmenus2[i].name = aa_fonts[i]->name;
+      fontmenus2[i].shortname = fontmenus[i].shortname;
+      fontmenus2[i].menuname = "aa_save2";
+      fontmenus2[i].key = NULL;
+      fontmenus2[i].type = MENU_INT;
+      fontmenus2[i].flags = UI;
+      fontmenus2[i].iparam = i;
+      fontmenus2[i].function = (void (*)(void)) aa_save2;
+    }
+  menu_add (fontmenus, nfonts);
+  menu_add (fontmenus2, nfonts);
+  for (i = 0; aa_formats[i] != NULL; i++);
+  nformats = i;
+  formatmenus = malloc (sizeof (menuitem) * i);
+  for (i = 0; aa_formats[i] != NULL; i++)
+    {
+      formatmenus[i].name = aa_formats[i]->formatname;
+      formatmenus[i].shortname = aa_formats[i]->formatname;
+      formatmenus[i].menuname = "aa_format";
+      formatmenus[i].key = NULL;
+      formatmenus[i].type = MENU_INT;
+      formatmenus[i].flags = UI;
+      formatmenus[i].iparam = i;
+      formatmenus[i].function = (void (*)(void)) aa_save;
+    }
+  menu_add (formatmenus, nformats);
+}
+static void
+aa_remove_menus ()
+{
+  int i;
+  menu_delnumbered (sizeof (ttext) / sizeof (char *), "palettemenu");
+  menu_delnumbered (AA_DITHERTYPES, "dither");
+  menu_delnumbered (MYNATTRS, "attribute");
+  menu_delnumbered (MYNATTRS, "save");
+  menu_delete (menuitems, NITEMS (menuitems));
+  menu_delete (fontmenus, nfonts);
+  menu_delete (fontmenus2, nfonts);
+  free (fontmenus2);
+  menu_delete (formatmenus, nformats);
+  free (formatmenus);
+  for (i = 0; aa_fonts[i] != NULL; i++)
+    {
+      free ((char *) fontmenus[i].shortname);
+    }
+  free (fontmenus);
+}
+static int cursorvisible = 1;
 static void
 aa_fflush (void)
 {
+  if (aa_cursorx < 0)
+    {
+      if (cursorvisible)
+	{
+	  aa_gotoxy (c, 0, 0);
+	  aa_hidecursor (c);
+	  cursorvisible = 0;
+	}
+    }
+  else
+    {
+      aa_gotoxy (c, aa_cursorx, aa_cursory);
+      if (!cursorvisible)
+	aa_showcursor (c), cursorvisible = 1;
+    }
   aa_flush (c);
 }
 
-static char *helptext[] =
-{
-  "AA DRIVER VERSION 1.0                  ",
-  "=====================                  ",
-  "                                       ",
-  "AA driver is currently the most        ",
-  "advanced and portable driver for XaoS  ",
-  "It is based at AAlib - an hight quality",
-  "ASCII-art library developed            ",
-  "by AA-project                          ",
-  "(see http://www.ta.jcu.cz/aa)          ",
-  "                                       ",
-  "It is fully featured XaoS driver for   ",
-  "text mode displays. It supports 256    ",
-  "colors and mouse where possible.       ",
-  "                                       ",
-  "It's advanced features is accesible via",
-  "Driver depended action menu availabele ",
-  "from varius stuff menu('V'). Menu      ",
-  "contain following stuff:               ",
-  " Attributes                            ",
-  "  AA-lib may use character attributes  ",
-  "  to reach fine qualitty of image.     ",
-  "  By default it uses normal, dim and   ",
-  "  bold characters where possible.      ",
-  "  But you should also enable different ",
-  "  ones like reversed or bold font      ",
-  "  characters. You may also enable usage",
-  "  of non ansii/reversed characters if  ",
-  "  your device supports it.             ",
-  " Font                                  ",
-  "  AA-lib uses bitmap image of font to  ",
-  "  prepare approximation table used for ",
-  "  ascii art rendering. This bitmap is  ",
-  "  expected to be same as one used by   ",
-  "  your device. AAlib performs          ",
-  "  detection where possible however some",
-  "  devices (like UNIX text terminals or ",
-  "  MDA does not support this.) AAlib has",
-  "  few font images compiled in, so in   ",
-  "  this cases you should try to use one ",
-  "  of them to archieve best results.    ",
-  " Inversion                             ",
-  "  Some devices uses inversed video     ",
-  "  use this to get correct colors at    ",
-  "  such devices.                        ",
-  " Dithering mode                        ",
-  "  Dithering is an way to get more exact",
-  "  color in approximation by combining  ",
-  "  more characters. But this method     ",
-  "  should bring ugly looking noise at   ",
-  "  certain images. Use this menu to     ",
-  "  disable or tune it.                  ",
-  " Palette opions                        ",
-  "  By default AA driver uses XaoS       ",
-  "  palette to render images, but        ",
-  "  it quite often looks ugly at text    ",
-  "  displays. Here you can choose special",
-  "  text palette instead. Note that with ",
-  "  filters enabled results should be    ",
-  "  rather ugly.                         ",
-  " Save text screen                      ",
-  "  Normal save function will generate   ",
-  "  normal PNG image instad of nice      ",
-  "  ascii-art. To save ascii art use this",
-  "  function instead. It supports many   ",
-  "  text file formats like HTML, ANSI,   ",
-  "  MORE, etc... It will also ask you for",
-  "  font and attributes(see above)       ",
-  "                                       ",
-  "                                       ",
-  " An AA driver was done by Jan Hubicka  ",
-  "                1997                   ",
-};
-#define UGLYTEXTSIZE (sizeof(helptext)/sizeof(char *))
-static struct params params[] =
-{
+static CONST struct params params[] = {
+  {"", P_HELP, NULL, "AA driver options:"},
   {"-aadriver", P_STRING, &aadriver, "Select display driver used by aa-lib"},
-  {"-kbddriver", P_STRING, &kbddriver, "Select keyboard driver used by aa-lib"},
-  {"-mousedriver", P_STRING, &mousedriver, "Select keyboard driver used by aa-lib"},
+  {"-kbddriver", P_STRING, &kbddriver,
+   "Select keyboard driver used by aa-lib"},
+  {"-mousedriver", P_STRING, &mousedriver,
+   "Select keyboard driver used by aa-lib"},
   {"-font", P_STRING, &deffont, "Select font"},
   {"-width", P_NUMBER, &width, "Set width"},
   {"-height", P_NUMBER, &height, "Set height"},
@@ -704,24 +658,33 @@ static struct params params[] =
   {"-recheight", P_NUMBER, &recheight, "Set recommended height"},
   {"-normal", P_SWITCH, enable, "enable usage of narmal characters"},
   {"-nonormal", P_SWITCH, disable, "disable usage of narmal characters"},
-  {"-dim", P_SWITCH, enable + 1, "enable usage of dim(half bright) characters"},
-  {"-nodim", P_SWITCH, disable + 1, "disable usage of dim(half bright) characters"},
-  {"-bold", P_SWITCH, enable + 2, "enable usage of bold(double bright) characters"},
-  {"-nobold", P_SWITCH, disable + 2, "disable usage of bold(double bright) characters"},
+  {"-dim", P_SWITCH, enable + 1,
+   "enable usage of dim(half bright) characters"},
+  {"-nodim", P_SWITCH, disable + 1,
+   "disable usage of dim(half bright) characters"},
+  {"-bold", P_SWITCH, enable + 2,
+   "enable usage of bold(double bright) characters"},
+  {"-nobold", P_SWITCH, disable + 2,
+   "disable usage of bold(double bright) characters"},
   {"-boldfont", P_SWITCH, enable + 3, "enable usage of boldfont characters"},
-  {"-noboldfont", P_SWITCH, disable + 3, "disable usage of boldfont characters"},
+  {"-noboldfont", P_SWITCH, disable + 3,
+   "disable usage of boldfont characters"},
   {"-reverse", P_SWITCH, enable + 4, "enable usage of reversed characters"},
-  {"-noreverse", P_SWITCH, disable + 4, "disable usage of reversed characters"},
+  {"-noreverse", P_SWITCH, disable + 4,
+   "disable usage of reversed characters"},
   {"-all", P_SWITCH, enable + 5, "enable usage of reserved characters"},
   {"-eight", P_SWITCH, enable + 6, "enable usage of non ansii characters"},
-{"-extended", P_SWITCH, &extended, "enable usage of extended character set"},
+  {"-extended", P_SWITCH, &extended,
+   "enable usage of extended character set"},
   {"-inverse", P_SWITCH, &inverse, "enable inverse"},
   {"-bright", P_NUMBER, &bright, "set bright (0-255)"},
   {"-contrast", P_NUMBER, &contrast, "set contrast (0-255)"},
   {"-gamma", P_FLOAT, &aa_gamma, "set famma (0-1)"},
   {"-nodither", P_SWITCH, dithering, "Disable dithering"},
-  {"-floyd_steinberg", P_SWITCH, dithering + 2, "Enable floyd steinberg dithering"},
-  {"-error_distribution", P_SWITCH, dithering + 1, "Enable error distribution dithering"},
+  {"-floyd_steinberg", P_SWITCH, dithering + 2,
+   "Enable floyd steinberg dithering"},
+  {"-error_distribution", P_SWITCH, dithering + 1,
+   "Enable error distribution dithering"},
   {"-random", P_NUMBER, &randomval, "Set random dithering value"},
   {"-dimmul", P_FLOAT, &dimmul, "Multiply factor for dim color (5.3)"},
   {"-boldmul", P_FLOAT, &boldmul, "Multiply factor for bold color (5.3)"},
@@ -729,8 +692,7 @@ static struct params params[] =
   {NULL, 0, NULL, NULL}
 };
 
-struct ui_driver aalib_driver =
-{
+struct ui_driver aalib_driver = {
   "aa",
   aa_initialize,
   aa_getsize,
@@ -744,21 +706,17 @@ struct ui_driver aalib_driver =
   aa_alloc_buffers,
   aa_free_buffers,
   aa_flip_buffers,
-  aa_clearscreen,
   aa_mousetype,
-  aa_action,
   aa_fflush,
   2,				/*text width */
   2,				/*text height */
-  helptext,
-  UGLYTEXTSIZE,
   params,
-  PALETTE_REDISPLAYS | SCREENSIZE | UPDATE_AFTER_RESIZE,	/*flags...see ui.h */
+  PALETTE_REDISPLAYS | SCREENSIZE | UPDATE_AFTER_RESIZE | AALIB,	/*flags...see ui.h */
   0.0, 0.0,			/*width/height of screen in centimeters */
   0, 0,				/*resolution of screen for windowed systems */
   UI_C256,			/*Image type */
   0, 255, 255			/*start, end of palette and maximum allocatable */
-				/*entries */
+    /*entries */
 };
 
 #endif
